from utils import NOC
import openpyxl as xl
import os

# Factors and their related points-- based on BCPNP policy guide
skill_level_points = {
    '00': 25,
    '0': 25,
    'A': 25,
    'B': 10,
    'C': 5,
    'D': 5
}
skill_level_bonus_points = {
    'NOC00': 15,
    'High Demand': 10,
    'Working in the position': 10
}

wage_points = [
    [100000, 1000000, 50],
    [97500, 99999, 38],
    [95000, 97499, 37],
    [92500, 94999, 36],
    [90000, 92499, 35],
    [87500, 89999, 34],
    [85000, 87499, 33],
    [82500, 84999, 32],
    [80000, 82499, 31],
    [77500, 79999, 30],
    [75000, 77499, 29],
    [72500, 74999, 28],
    [70000, 72499, 27],
    [67500, 69999, 26],
    [65000, 67499, 25],
    [62500, 64999, 24],
    [60000, 62499, 23],
    [57500, 59999, 22],
    [55000, 57499, 21],
    [52500, 54999, 20],
    [50000, 52499, 19],
    [47500, 49999, 18],
    [45000, 47499, 17],
    [42500, 44999, 16],
    [40000, 42499, 15],
    [38750, 39999, 14],
    [37500, 38749, 13],
    [36250, 37499, 12],
    [35000, 36249, 11],
    [33750, 34999, 10],
    [32500, 33749, 9],
    [31250, 32499, 8],
    [30000, 31249, 7],
    [28750, 29999, 6],
    [27500, 28749, 5],
    [26250, 27499, 4],
    [25000, 26249, 3],
    [0, 25000, 0]
]
region_points = [
    [
        'Stikine, Central Coast, Northern Rockies, Mount Waddington, Skeena-Queen Charlotte, Powell River, Sunshine Coast, Kootenay-Boundary, Alberni-Clayoquot',
        10],
    ['Kitimat-Stikine, Bulkley-Nechako, Squamish-Lillooet, Strathcona, Columbia-Shuswap, East Kootenay', 8],
    ['Peace River, Comox Valley, Cariboo, Central Kootenay', 6],
    ['Okanagan-Similkameen, Cowichan Valley, North Okanagan, Fraser-Fort George', 4],
    ['Thompson-Nicola, Nanaimo, Central Okanagan', 2],
    ['Capital, Fraser Valley', 2],
    ['Greater Vancouver', 0]

]
# this is based on months
work_experience_points = [
    [60, 120, 15],
    [48, 59, 12],
    [36, 47, 9],
    [24, 35, 6],
    [12, 23, 3],
    [1, 11, 1],
    [0, 0, 0]
]
education_points = {
    'Doctor': 17,
    'Master': 17,
    'Postgraduate': 11,
    'Bachelor': 11,
    'Trade Diploma': 11,
    'Associate Degree': 4,
    'Non-trade Diploma': 2,
    'Secondary': 0
}
education_bonus_points = {
    'BC Post-secondary': 8,
    'Canada Post-secondary': 6,  # Outside of BC
    'ECA': 4,  # with Education Credential Assessment
    'Trade certificate': 4,
    # Successfully completed the Industry Training Authority British Columbia (ITABCs) challenge certification process
    'Nothing': 0
}
language_points = {
    10: 30,
    9: 26,
    8: 22,
    7: 18,
    6: 14,
    5: 10,
    4: 6,
    3: 0
}


def getWagePoints(wage):
    if wage > 1000000:
        wage = 1000000
    elif wage <= 0:
        wage = 0
    for i in range(len(wage_points)):
        if wage in range(wage_points[i][0], wage_points[i][1]):
            return (wage_points[i][2])


# region points based on the index of the list
def getRegionPoints(region_index):
    return region_points[region_index][1]


def getWorkExperiencePoints(month):
    if month > 120:
        month = 120
    elif month <= 0:
        month = 0
    for i in range(len(work_experience_points)):
        if month in range(work_experience_points[i][0], work_experience_points[i][1]):
            return (work_experience_points[i][2])


# Passing in education  and education bonus: text. Return a list containing edu points and bonus points
def getEducationPoints(education, education_bonus):
    edu_points = education_points.get(education)
    bonus_points = education_bonus_points.get(education_bonus)
    return [edu_points, bonus_points]


def getLanguagePoints(clb_level):
    if clb_level > 10:
        clb_level = 10
    elif clb_level < 4:
        clb_level = 3
    return language_points.get(clb_level)


def getSkillLevelPoints(noc):
    sub_skill_level_NOC00_points = 0
    sub_skill_lelve_high_demand_points = 0
    sub_skill_level_points = 0

    the_noc_level = NOC.getNOCLevel(noc)
    sub_skill_level_points = skill_level_points.get(the_noc_level)

    if the_noc_level == '00':
        sub_skill_level_NOC00_points = skill_level_bonus_points['NOC00']
    if isHighDemand(noc):
        sub_skill_lelve_high_demand_points = skill_level_bonus_points['High Demand']

    points = [sub_skill_level_points, sub_skill_level_NOC00_points, sub_skill_lelve_high_demand_points]
    return points


def is_Working_one_year(is_work_in_the_position, has_one_year_canadian_experience):
    sub_skill_level_work_in_position_points = 0
    sub_skill_level_has_one_year_canadian_experience_points = 0

    if is_work_in_the_position:
        sub_skill_level_work_in_position_points = skill_level_bonus_points['Working in the position']
    if has_one_year_canadian_experience:
        sub_skill_level_has_one_year_canadian_experience_points = 10
    points = [sub_skill_level_work_in_position_points, sub_skill_level_has_one_year_canadian_experience_points]
    return points


# Since the high demand occupations are subject to be changed ofter, so we read it from excel file
# def isHighDemand(noc):
#     is_high_demand = False
#     file = os.getcwd() + '\BCPNP Skills Policy.xlsx'
#     try:
#         wb = xl.load_workbook(file)
#     except OSError:
#         print(f'cannot open {file}')
#
#     sheet = wb['High Demand']
#
#     for row in range(2, sheet.max_row):

#         if noc == str(sheet.cell(row, 5).value):
#             is_high_demand = True
#
#     return is_high_demand

def isHighDemand(noc):
    high_demand = [
        '0621', '0631', '0013', '0714', '0213', '0111', '0601', '0122', '0016', '0124', '0015', '0121', '0821', '0731',
        '0712', '0632', '0114', '0423', '0112', '0014', '0125', '0651', '0422', '0113', '0421', '0211', '0012', '0513',
        '0512', '0912', '0212', '0412', '0414', '1111', '4032', '2171', '2174', '1114', '4112', '1122', '2173', '1123',
        '4021', '2131', '4011', '4163', '1121', '5131', '4152', '4153', '2175', '5121', '4165', '2151', '1112', '4161',
        '1113', '2147', '4151', '1221', '4212', '1311', '1241', '4214', '6232', '7321', '5241', '6341', '5254', '6221',
        '1224', '4311', '6231', '6235', '2281', '2242', '6211', '1215', '1242', '1222', '2282', '1243', '2271', '4211',
        '1312', '4312', '7315', '1223', '1212', '5221', '4313', '1315', '1414', '6513', '4412', '4411', '7513', '1513',
        '1512', '1511', '3112', '3413', '3142', '3143', '3233', '3012', '3216', '3234'
    ]
    is_high_demand = True if noc in high_demand else False
    return is_high_demand


def BCPoints(applicant):
    sub_skill_level_points = getSkillLevelPoints(applicant.noc)
    is_working_one_year_points = is_Working_one_year(applicant.is_working_in_the_position,
                                                     applicant.has_one_year_canadian_experience)
    sub_wage_points = getWagePoints(
        float(applicant.hourly_rate) * float(applicant.working_hours) * 52)  # annual salary
    sub_region_points = getRegionPoints(applicant.region)
    sub_work_experience_points = getWorkExperiencePoints(applicant.work_experience)
    sub_education_points = getEducationPoints(applicant.education, applicant.education_bonus)
    sub_language_points = getLanguagePoints(applicant.language[0].clb)

    total_points = is_working_one_year_points[0] + is_working_one_year_points[1] + sub_skill_level_points[0] + \
                   sub_skill_level_points[1] + sub_skill_level_points[
                       2] + sub_wage_points + sub_region_points + sub_work_experience_points + sub_education_points[0] + \
                   sub_education_points[1] + sub_language_points
    print(f'Total points: {total_points}')

    # return a dictionary including factors and relative points
    points = {
        'total_points': total_points,
        'noc_level_points': sub_skill_level_points[0],  # NOC level points
        'noc_00_points': sub_skill_level_points[1],  # NOC 00 points
        'top_100_points': sub_skill_level_points[2],  # Top 100 points
        'working_in_the_position': is_working_one_year_points[0],  # Working in the position
        'one_year_Canadian_work_experience': is_working_one_year_points[1],
        # Has one year Canadian work experience in the position
        'wage_points': sub_wage_points,
        'region_points': sub_region_points,
        'work_experience_points': sub_work_experience_points,
        'dducaiton_level_points': sub_education_points[0],  # Frit argument: points of level of education
        'education_bonus_points': sub_education_points[1],  # Second argument: points of 4 possible situations
        'language_level_points': sub_language_points,
    }
    return points
