from models.applicant import Person
from bcPoints import bcScoring
from ee import fswScoring
from ee import eeScoring
from skPoints import skScoring
from nsPoints import nsScoring
from utils.language import *


# 1. test fsw eligibility
def testFSW():
    fsw_applicant = Person()

    fsw_applicant.first_language = [455, 11, 504, 12]
    fsw_applicant.first_language_format = 'TCF'
    fsw_applicant.second_language = [6, 6, 6, 6]
    fsw_applicant.second_language_format = 'IELTS'
    fsw_applicant.education = 'Master'
    fsw_applicant.work_experience = 5
    fsw_applicant.age = 35
    fsw_applicant.aeo = True
    fsw_applicant.adaptability = {
        "Spouse language": 4,
        "Studied in Canada": False,
        "Spouse studied in Canada": False,
        "Worked in Canada": False,
        "Spouse worked in Canada": False,
        "AEO": False,
        "Relatives in Canada": False
    }

    points = fswScoring.fswPoints(fsw_applicant)
    print(points)


# test BCPNP scoring
def testBCPNP():
    applicant = Person()
    applicant.name = "Jackson"
    applicant.noc = '1241'
    applicant.hourly_rate = 17
    applicant.working_hours = 40
    applicant.is_working = True
    applicant.region = 2
    applicant.has_one_year_canadian_experience = True
    applicant.is_working_in_the_position = True
    applicant.education = 'Bachelor'
    applicant.education_bonus = 'BC Post-secondary'
    applicant.language = 10
    applicant.years_work_experience = 3

    points = bcScoring.BCPoints(applicant)
    print(points)


# test EE scoring
def testEE():
    ee_applicant = Person()
    ee_applicant.with_spouse = True
    ee_applicant.age = 33
    ee_applicant.education = 'Master'
    ee_applicant.first_language = [455, 11, 504, 12]
    ee_applicant.first_language_format = 'TCF'
    ee_applicant.second_language = [6, 6, 6, 6]
    ee_applicant.second_language_format = 'IELTS'
    ee_applicant.canadian_work_experience = 5
    ee_applicant.with_trade_certification = True
    ee_applicant.foreign_work_experience = 5
    ee_applicant.canadian_relative = 'brother'
    ee_applicant.education_country = 'Canada'
    ee_applicant.aeo = True
    ee_applicant.noc = '0013'
    ee_applicant.pnp_nominated = False

    ee_spouse = Person()
    ee_spouse.education = "Secondary"
    ee_spouse.language = [6, 6, 6, 6]
    ee_spouse.language_format = 'IELTS'
    ee_spouse.ca_we = 3
    ee_spouse.canadian_relative = 'sister'

    print(eeScoring.getEEPoints(ee_applicant, ee_spouse))


# test SKPNP
def testSKPNP():
    applicant = Person()
    applicant.education = 'Master'
    applicant.we_within5 = 3
    applicant.we_within6_10 = 4

    applicant.language = [6, 6, 6, 6]
    applicant.language_format = 'IELTS'
    applicant.clb, applicant.clbs = getCLBs(applicant.language, applicant.language_format)

    applicant.age = 34


    applicant.sk_connection = {
        'Has approved job offer': True,
        'Relative': 'sibling',
        'Work years in SK': 3,
        'Study academic years in SK': 1
    }

    print(skScoring.SKPoints(applicant))

# test NSNP
def testNSNP():
    applicant = Person()
    applicant.education = '3-year post-secondary'
    applicant.first_language = [6, 6, 6, 6]
    applicant.first_language_format ='IELTS'
    applicant.second_language = [] #[455, 11, 504, 12]
    applicant.second_language_format = 'TCF'
    applicant.work_years=5   # with past 6 years
    applicant.age = 30
    applicant.aeo=True
    applicant.studied_in_NS=False
    applicant.worked_in_NS=False
    applicant.relative_in_NS=''

    spouse=Person()
    spouse.studied_in_NS=False
    spouse.worked_in_NS = False
    spouse.relative_in_NS=''
    spouse.language=[5,5,5,5]
    spouse.language_format = 'IELTS'
    spouse.clb, spouse.clbs=getCLBs(spouse.language,spouse.language_format)
    spouse.language_bonus=True if spouse.clb>=4 else False

    print(nsScoring.NSPoints(applicant,spouse))

# print('The FSW points:')
# testFSW()
# print('\n')
# print('The BCPNP points:')
# print('\n')
# testBCPNP()
# print('The EE points:')
# testEE()
# print('\n')
# print('The SKPNP points:')
# testSKPNP()
print('\n')
testNSNP()

