from utils.language import *
from utils.NOC import *

# Age points here
age_points_table = (
    [18, 90, 99],
    [19, 95, 105],
    [30, 95, 105],
    [31, 90, 99],
    [32, 85, 94],
    [33, 80, 88],
    [34, 75, 83],
    [35, 70, 77],
    [36, 65, 72],
    [37, 60, 66],
    [38, 55, 61],
    [39, 50, 55],
    [40, 45, 50],
    [41, 35, 39],
    [42, 25, 28],
    [43, 15, 17],
    [44, 5, 6],
)


def getAgePoints(age, with_spouse):
    if age < 18 or age > 44:
        return 0
    elif age in range(20, 30):
        if with_spouse:
            return 100
        else:
            return 110
    else:
        for row in range(0, len(age_points_table)):
            if age == age_points_table[row][0]:
                if with_spouse:
                    return age_points_table[row][1]
                else:
                    return age_points_table[row][2]


# Education points here
edu_level_points_table = {
    'Doctor': 150,
    'Master': 135,
    'Professional': 135,
    # professional degree needed to practice in a licensed profession. https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after
    'Two more post-secondary': 128,  # Two or more post-secondary, at least one is more than 3 year
    '3-year post-secondary': 120,
    '2-year post-secondary': 98,
    '1-year post-secondary': 90,
    'Secondary': 30
}

edu_level_points_table_with_spouse = {
    'Doctor': 140,
    'Master': 126,
    'Professional': 126,
    # professional degree needed to practice in a licensed profession. https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after
    'Two more post-secondary': 119,  # Two or more post-secondary, at least one is more than 3 year
    '3-year post-secondary': 112,
    '2-year post-secondary': 91,
    '1-year post-secondary': 84,
    'Secondary': 28
}


def getEduPoints(education, with_spouse):
    if with_spouse:
        return edu_level_points_table_with_spouse.get(education)
    else:
        return edu_level_points_table.get(education)


# Language points here
clb_points_table = (
    [10, 32, 34],
    [9, 29, 31],
    [8, 22, 23],
    [7, 16, 17],
    [6, 8, 9],
    [5, 6, 6],
    [4, 6, 6],

)


def get1stLanguagePoints(language, with_spouse):
    clb = language.clb
    if clb > 10:
        clb = 10
    if clb < 4:
        return 0
    for row in range(0, len(clb_points_table)):
        if clb == clb_points_table[row][0]:
            if with_spouse:
                return clb_points_table[row][1]
            else:
                return clb_points_table[row][2]


# Second official language (maximum 4 points)
clb_points_table2 = (
    [5, 1, 1],
    [6, 1, 1],
    [7, 3, 3],
    [8, 3, 3],
    [9, 6, 6]
)


def get2ndLanguagePoints(language, with_spouse):
    clb = language.clb
    if clb < 5:
        return 0
    if clb > 9:
        clb = 9
    for row in range(0, len(clb_points_table2)):
        if clb == clb_points_table2[row][0]:
            if with_spouse:
                return clb_points_table2[row][1]
            else:
                return clb_points_table2[row][2]


def getLanguagePoints(language, with_spouse=False):
    if len(language) == 0:
        return 0
    if language[0] is None:
        return 0
    elif language[1] is None:
        clb_points = get1stLanguagePoints(language[0], with_spouse)
    else:
        clb_points = get1stLanguagePoints(language[0], with_spouse) + get2ndLanguagePoints(language[1], with_spouse)
    return clb_points


# Canadian work experience points here
canadian_work_experience_points_table = (
    [1, 35, 40],
    [2, 46, 53],
    [3, 56, 64],
    [4, 63, 72],
    [5, 70, 80]
)


def getCanadianWEPoints(years, with_spouse):
    if years < 1:
        return 0
    if years > 5:
        years = 5
    for row in range(0, len(canadian_work_experience_points_table)):
        if years == canadian_work_experience_points_table[row][0]:
            if with_spouse:
                return canadian_work_experience_points_table[row][1]
            else:
                return canadian_work_experience_points_table[row][2]


spouse_edu_points_table = {
    'Doctor': 10,
    'Master': 10,
    'Professional': 10,
    # professional degree needed to practice in a licensed profession. https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after
    'Two more post-secondary': 9,  # Two or more post-secondary, at least one is more than 3 year
    '3-year post-secondary': 8,
    '2-year post-secondary': 7,
    '1-year post-secondary': 6,
    'Secondary': 2
}


def getSpouseEduPoints(spouse_edu):
    return spouse_edu_points_table.get(spouse_edu)


spouse_language_points_talbe = {
    5: 1,
    6: 1,
    7: 3,
    8: 3,
    9: 5
}


def getSpouseLanguagePoints(language):
    clb = language.clb
    return spouse_language_points_talbe.get(clb)


spouse_ca_we_points_table = {
    1: 5,
    2: 7,
    3: 8,
    4: 9,
    5: 10
}


def getSpouseCaWePoints(years):
    if years > 5:
        years = 5
    if years < 1:
        return 0
    return spouse_ca_we_points_table.get(years)


def getSpousePoints(spouse):
    edu_points = getSpouseEduPoints(spouse.education)
    language_points = getSpouseLanguagePoints(spouse.language)
    ca_we_points = getSpouseCaWePoints(spouse.canadian_work_experience)
    sub_total = edu_points + language_points + ca_we_points

    in_details = {
        'Spouse education points': edu_points,
        'Spouse language points': language_points,
        'Spouse Canadian work experience points': ca_we_points
    }
    return sub_total, in_details


# With good official language proficiency (Canadian Language Benchmark Level [CLB] 7 or higher) and a post-secondary degree
# Points for CLB 7 or more on all first official language abilities, with one or more under CLB 9 (Maximum 25 points)
# Points for CLB 9 or more on all four first official language abilities (Maximum 50 points)
edu_language7_points_table = {
    'Doctor': 25,
    'Master': 25,
    'Professional': 25,
    'Two more post-secondary': 25,
    '3-year post-secondary': 13,
    '2-year post-secondary': 13,
    '1-year post-secondary': 13,
    'Secondary': 0
}

edu_language9_points_table = {
    'Doctor': 50,
    'Master': 50,
    'Professional': 50,
    'Two more post-secondary': 50,
    '3-year post-secondary': 25,
    '2-year post-secondary': 25,
    '1-year post-secondary': 25,
    'Secondary': 0
}


def getEduLanguagePoints(education, language):
    clb = language.clb
    if clb < 7:
        return 0
    elif clb >= 9:
        return edu_language9_points_table.get(education)
    else:
        return edu_language7_points_table.get(education)


# With Canadian work experience and a post-secondary degree
# Points for education + 1 year of Canadian work experience (Maximum 25 points)
# Points for education + 2 years or more of Canadian work experience (Maximum 50 points)
edu_ca_we1_points_table = {
    'Doctor': 25,
    'Master': 25,
    'Professional': 25,
    'Two more post-secondary': 25,
    '3-year post-secondary': 13,
    '2-year post-secondary': 13,
    '1-year post-secondary': 13,
    'Secondary': 0
}

edu_ca_we2_points_table = {
    'Doctor': 50,
    'Master': 50,
    'Professional': 50,
    'Two more post-secondary': 50,
    '3-year post-secondary': 25,
    '2-year post-secondary': 25,
    '1-year post-secondary': 25,
    'Secondary': 0
}


def getEduCaWePoints(education, years):
    if years >= 2:
        return edu_ca_we2_points_table.get(education)
    elif years >= 1:
        return edu_ca_we1_points_table.get(education)
    else:
        return 0


# Foreign work experience – With good official language proficiency (Canadian Language Benchmark Level [CLB] 7 or higher)
# Points for foreign work experience + CLB 7 or more on all first official language abilities, one or more under 9 (Maximum 25 points)
# Points for foreign work experience + CLB 9 or more on all four first official language abilities (Maximum 50 points)
foreign_we_languange7_points_table = {
    1: 13,
    2: 13,
    3: 25
}

foreign_we_languange9_points_table = {
    1: 25,
    2: 25,
    3: 50
}


def getForeignWeLanguagePoints(years, language):
    clb = language.clb
    if years > 3:
        years = 3
    if clb < 7:
        return 0
    elif clb >= 9:
        return foreign_we_languange9_points_table.get(years)
    else:
        return foreign_we_languange7_points_table.get(years)


# Foreign work experience – With Canadian work experience
# Points for foreign work experience + 1 year of Canadian work experience (Maximum 25 points)
# Points for foreign work experience + 2 years or more of Canadian work experience (Maximum 50 points)

foreign_we_ca1_we_points_table = {
    1: 13,
    2: 13,
    3: 25
}

foreign_we_ca2_we_points_table = {
    1: 25,
    2: 25,
    3: 50
}


def getForeignWeCaWePoints(foreign_years, ca_years):
    if foreign_years > 3:
        foreign_years = 3
    if ca_years >= 2:
        return foreign_we_ca2_we_points_table.get(foreign_years)
    elif ca_years >= 1:
        return foreign_we_ca1_we_points_table.get(foreign_years)
    else:
        return 0


# Certificate of qualification (trade occupations) – With good official language proficiency (Canadian Language Benchmark Level [CLB] 5 or higher)
# Points for certificate of qualification + CLB 5 or more on all first official language abilities, one or more under 7 (Maximum 25 points)
# Points for certificate of qualification + CLB 7 or more on all four first official language abilities (Maximum 50 points)

def getTradeBonus(language):
    clb = language.clb
    if clb >= 7:
        return 50
    elif clb >= 5:
        return 25
    else:
        return 0


def getAdditionalPoints(applicant, spouse=None):
    # 1. Brother or sister living in Canada who is a citizen or permanent resident of Canada	15
    relatives = ['brother', 'sister']
    if (applicant.canadian_relative.lower() in relatives) or (spouse.canadian_relative.lower() in relatives):
        relative_points = 15
    else:
        relative_points = 0

    # Scored NCLC 7 or higher on all four French language skills and scored CLB 4 or lower in English (or didn’t take an English test)	15
    # Scored NCLC 7 or higher on all four French language skills and scored CLB 4 or lower in English (or didn’t take an English test)	15
    # Scored NCLC 7 or higher on all four French language skills and scored CLB 5 or higher on all four English skills	30
    if applicant.language[0].format == 'TEF' or applicant.language[0].format == 'TCF':
        clb1 = applicant.language[0].clb
        clb2 = applicant.language[1].clb
        if clb1 >= 7:
            if clb2 >= 5:
                nclc_points = 30
            else:
                nclc_points = 15
        else:
            nclc_points = 0
    elif applicant.language[1].format == 'TEF' or applicant.language[1].format == 'TCF':
        clb1 = applicant.language[1].clb
        clb2 = applicant.language[0].clb
        if clb1 >= 7:
            if clb2 >= 5:
                nclc_points = 30
            else:
                nclc_points = 15
        else:
            nclc_points = 0
    else:
        nclc_points = 0

    # Post-secondary education in  Canada - credential of one or two years	15
    # Post-secondary education in  Canada - credential three years or longer	30

    if (applicant.education_country == 'Canada'):
        if applicant.education in ['Two more post-secondary', '1-year post-secondary']:
            edu_in_canada_points = 15
        elif applicant.education in ['Doctor', 'Master', 'Professional', 'Two more post-secondary',
                                     '3-year post-secondary']:
            edu_in_canada_points = 30
        else:
            edu_in_canada_points = 0
    else:
        edu_in_canada_points = 0

    # Arranged employment - NOC 00	200
    # Arranged employment – any other NOC 0, A or B	50
    if applicant.aeo:
        noc = getNOCLevel(applicant.noc)
        if noc == '00':
            aeo_points = 200
        elif noc in ['0', 'A', 'B']:
            aeo_points = 50
        else:
            aeo_points = 0
    else:
        aeo_points = 0

    # Provincial or territorial nomination	600
    if applicant.pnp_nominated:
        pnp_points = 600
    else:
        pnp_points = 0

    sub_total = relative_points + nclc_points + edu_in_canada_points + aeo_points + pnp_points

    additional_points = {
        'Relative points': relative_points,
        'NCLC points': nclc_points,
        'Education in Canada points': edu_in_canada_points,
        'AEO points': aeo_points,
        'PNP nomination points': pnp_points
    }
    return sub_total, additional_points


def getEEPoints(applicant, spouse=None):
    ee_age_points = getAgePoints(applicant.age, applicant.with_spouse)
    ee_edu_points = getEduPoints(applicant.ee_edu_level, applicant.with_spouse)
    ee_language_points = getLanguagePoints(applicant.language, applicant.with_spouse)
    ee_canadian_work_experience_points = getCanadianWEPoints(applicant.canadian_work_experience,
                                                             applicant.with_spouse)
    # if with spouse, calculate the spouse points
    if applicant.with_spouse:
        ee_spouse_points, ee_spouse_points_in_detail = getSpousePoints(spouse)
    else:
        ee_spouse_points = 0
        ee_spouse_points_in_detail = None

    edu_language_points = getEduLanguagePoints(applicant.ee_edu_level, applicant.language[0])
    edu_ca_we_points = getEduCaWePoints(applicant.ee_edu_level, applicant.canadian_work_experience)
    foreign_we_lanuage_points = getForeignWeLanguagePoints(applicant.foreign_work_experience, applicant.language[0])
    foreign_we_ca_we_points = getForeignWeCaWePoints(applicant.foreign_work_experience,
                                                     applicant.canadian_work_experience)
    # if Certificate of qualification (trade occupations)
    if applicant.with_trade_certification:
        ee_trade_language_points = getTradeBonus(applicant.language[0])
    else:
        ee_trade_language_points = 0

    additioanlPoints, additionalinDetails = getAdditionalPoints(applicant, spouse)

    total_points = ee_age_points + ee_edu_points + ee_language_points + ee_canadian_work_experience_points + ee_spouse_points + edu_language_points + edu_ca_we_points + foreign_we_lanuage_points + foreign_we_ca_we_points + ee_trade_language_points + additioanlPoints
    points = {
        'total_point': total_points,
        'age_points': ee_age_points,
        'education_points': ee_edu_points,
        'language_points': ee_language_points,
        'canadian_work_experience_points': ee_canadian_work_experience_points,
        'spouse_points': ee_spouse_points,
        'spouse_points_detail': ee_spouse_points_in_detail,
        'education_language_points': edu_language_points,
        'education_canadian_work_experience_points': edu_ca_we_points,
        'foreign_work_experience_language_points': foreign_we_lanuage_points,
        'foreign_work_experience_canadian_work_experience_points': foreign_we_ca_we_points,
        'trade_certification_language': ee_trade_language_points,
        'additional_points': additioanlPoints,
        'additoional_points_in_detail': additionalinDetails

    }
    return points
