from utils.language import *

# First official language (maximum 24 points)
# Check the table below and add the points that match your skill level:
#
# Points
# First official language
#               S	L	R	W
# CLB level 9+ 	6	6	6	6
# CLB level 8	5	5	5	5
# CLB level 7	4	4	4	4
# CLB level 7-  Not eligible

clb_level_points_talbe = {
    9: 6,
    8: 5,
    7: 4
}


def get1stLanguagePoints(language):
    clb, clbs = language.clb, [language.clb_r, language.clb_w, language.clb_l, language.clb_s]
    if clb < 7:
        print('the client is not eligible')
    elif clbs != []:
        points = []
        for clb in clbs:
            if clb > 9:
                clb = 9
            sub_points = clb_level_points_talbe.get(clb)
            points.append(sub_points)
        return points[0] + points[1] + points[2] + points[3]


# Second official language (maximum 4 points)
# You can get 4 points only if you have a score of at least CLB 5 in each of the 4 language abilities.
#
# Second official language	Points
# At least CLB 5 in all of the 4 abilities	4
# CLB 4 or less in any of the 4 abilities	0

def get2ndLanguagePoints(language):
    clb, clbs = language.clb, [language.clb_r, language.clb_w, language.clb_l, language.clb_s]
    if clbs[0] >= 5 & clbs[1] >= 5 & clbs[2] >= 5 & clbs[3] >= 5:
        return 4
    else:
        return 0


# Education points get here
# https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after

edu_level_points_table = {
    'Doctor': 25,
    'Master': 23,
    'Professional': 23,
    # professional degree needed to practice in a licensed profession. https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after
    'Two more post-secondary': 22,
    # Two or more post-secondary program credentials, at least one of which must have been issued on completion of a post-secondary program of three years or longer [R78(1)(e)]
    '3-year post-secondary': 21,
    '2-year post-secondary': 19,
    '1-year post-seconary': 15,
    'Secondary': 5
}


def getEduPoints(education):
    return edu_level_points_table.get(education)


work_experience_points_table = {
    1: 9,
    2: 11,
    3: 11,
    4: 13,
    5: 13,
    6: 15
}


def getWorkExperiencePoints(years):
    if not isinstance(years, int):
        print("Years must be integer")
        return 0
    if years > 6:
        years = 6
    if years < 1:
        return 0
    return work_experience_points_table.get(years)


# Age points starts here
age_points_table = {
    # under 18 and above 47, return 0 directly in function
    # 18-35 return 12 directly in function
    36: 11,
    37: 10,
    38: 9,
    39: 8,
    40: 7,
    41: 6,
    42: 5,
    43: 4,
    44: 3,
    45: 2,
    46: 1
}


def getAgePoints(age):
    if age < 18 or age >= 47:
        return 0
    elif age in range(18, 36):
        return 12
    else:
        return age_points_table.get(age)


# Arranged employment points starts here
# if has valid job offer, such as LMIA supported work permit and job offer, varies valid_job_offer will be tree
def getAeoPoints(valid_job_offer):
    if valid_job_offer:
        return 10


# Adaptability points starts here
# Policy refers to: https://www.canada.ca/en/immigration-refugees-citizenship/services/immigrate-canada/express-entry/eligibility/federal-skilled-workers/six-selection-factors-federal-skilled-workers.html#experience

def getAdaptabilityPoints(adaptability):
    points = 0
    if adaptability.get('Spouse language') >= 4:
        points += 5
    if adaptability.get('Studied in Canada'):
        points += 5
    if adaptability.get('Spouse studied in Canada'):
        points += 5
    if adaptability.get('Worked in Canada'):
        points += 5
    if adaptability.get('Spouse worked in Canada'):
        points += 5
    if adaptability.get('AEO'):
        points += 5
    if adaptability.get('Relatives in Canada'):
        points += 5

    if points > 10:
        return 10
    else:
        return points


# public interface.
def fswPoints(fsw):
    if len(fsw.language) > 1:
        clb_points = get1stLanguagePoints(fsw.language[0]) + get2ndLanguagePoints(fsw.language[1])
    else:
        clb_points = get1stLanguagePoints(fsw.language[0])
    edu_points = getEduPoints(fsw.education.ee_level)
    work_points = getWorkExperiencePoints(fsw.work_experience.years)
    age_points = getAgePoints(fsw.age)
    aeo_points = getAeoPoints(fsw.aeo)
    adaptability_points = getAdaptabilityPoints(fsw.adaptability)

    total_points = clb_points + edu_points + age_points + aeo_points + adaptability_points

    # print(f'clb points:',clb_points)
    # print(f'edu points:', edu_points)
    # print(f'work points:', work_points)
    # print(f'age points:', age_points)
    # print(f'aeo points:', aeo_points)
    # print(f'adaptability points:', adaptability_points)
    # print(f'total points:', total_points)

    points = {
        "Total points": total_points,
        'Language points': clb_points,
        "Education points": edu_points,
        "Work experience points": work_points,
        "Age points": age_points,
        "AEO points": aeo_points,
        "adaptability points": adaptability_points
    }

    return points

def expfswPoints(fsw):
    if len(fsw.language) > 1:
        clb_points = get1stLanguagePoints(fsw.language[0]) + get2ndLanguagePoints(fsw.language[1])
    else:
        clb_points = get1stLanguagePoints(fsw.language[0])
    edu_points = getEduPoints(fsw.ee_edu_level)
    work_points = getWorkExperiencePoints(fsw.work_experience)
    age_points = getAgePoints(fsw.age)
    aeo_points = getAeoPoints(fsw.aeo)
    adaptability_points = getAdaptabilityPoints(fsw.adaptability)

    total_points = clb_points + edu_points + age_points + aeo_points + adaptability_points

    # print(f'clb points:',clb_points)
    # print(f'edu points:', edu_points)
    # print(f'work points:', work_points)
    # print(f'age points:', age_points)
    # print(f'aeo points:', aeo_points)
    # print(f'adaptability points:', adaptability_points)
    # print(f'total points:', total_points)

    points = {
        "total_points": total_points,
        'language_points': clb_points,
        "education_points": edu_points,
        "work_experience_points": work_points,
        "age_points": age_points,
        "aeo_points": aeo_points,
        "adaptability_points": adaptability_points
    }

    return points