# from django.db import models
# from django.db.models import Model
from utils.others import age, Sex
from utils.language import getCLBs
from datetime import datetime
from dateutil import relativedelta



class Person:
    def __init__(self, first_name='', last_name='', dob='', sex=Sex.Female):
        self.first_name = first_name
        self.last_name = last_name
        self.full_name = self.first_name + ' ' + self.last_name
        self.DOB = datetime.strptime(dob, '%Y-%m-%d')
        self.sex = sex
        self.sex_text = 'male' if self.sex else 'female'
        self.age = age(self.DOB)

    def __str__(self):
        return f'Person object (name:{self.full_name})'

    def __call__(self, *args, **kwargs):
        return f'My name is {self.full_name}, {self.sex_text}, and I am {self.age} years old'


# p=Person('Jacky','Zhang','1969-02-11',Sex.Male)
# print(p())

# Application
class Application:
    def __init__(self, application):
        self.application = application


# person_id application，
# factors (language,education,work_experience,age,adaptability,job offer,spouse,relative)
class Solution:
    pass



class Language:
    def __init__(self, language):
        self.reading = language[0]
        self.writing = language[1]
        self.listening = language[2]
        self.speaking = language[3]
        self.format = language[4]
        self.clb, [self.clb_r, self.clb_w, self.clb_l, self.clb_s] = getCLBs(
            [self.reading, self.writing, self.listening, self.speaking], self.format)


class Education:
    def __init__(self, start_date, end_date, program_years, level, study_field, name_of_institution, city, province,
                 country):
        self.start_date = datetime.strptime(start_date, '%Y-%m-%d')
        self.end_date = datetime.strptime(end_date, '%Y-%m-%d')
        self.program_years = program_years
        self.level = level
        self.study_field = study_field
        self.name_of_institution = name_of_institution
        self.city = city
        self.province = province
        self.country = country
        self.is_Canadian = True if self.country == 'Canada' else False
        if self.level=='Bachelor':
            self.ee_level='3-year post-secondary'

    def in_province(self, province):
        return True if self.province == province else False

    def __str__(self):
        return f'Education object (name:{self.level} in {self.name_of_institution})'

    def __call__(self, *args, **kwargs):
        return f'It is a {self.program_years}-year {self.level} of {self.study_field} in {self.name_of_institution},{self.city},{self.province},{self.country},which started form {self.start_date} and ended on {self.end_date}'


# s = datetime.strptime('2005-05-31', '%Y-%m-%d')
# e = datetime.strptime('2007-06-30', '%Y-%m-%d')
# edu = Education(s, e, 2, 'Master', 'MBA', 'NJU', 'Nanjing', 'Jiangsu', 'China')
# print(edu())

class WorkExperience:
    def __init__(self, start_date, end_date, title, noc, employer, city, province, country):
        self.start_date = datetime.strptime(start_date, '%Y-%m-%d')
        self.end_date = datetime.strptime(end_date, '%Y-%m-%d')
        self.title = title
        self.noc = noc
        self.employer = employer
        self.city = city
        self.province = province
        self.country = country
        self.years = relativedelta.relativedelta(self.end_date, self.start_date).years
        self.months = relativedelta.relativedelta(self.end_date, self.start_date).months
        self.days = relativedelta.relativedelta(self.end_date, self.start_date).days
        self.is_Canadian = True if self.country == 'Canada' else False

    def in_province(self, province):
        return True if self.province == province else False

    def __str__(self):
        return f'Work experience object (name:{self.title} in {self.employer})'

    def __call__(self, *args, **kwargs):
        s1 = f'It is a {self.years}-year work experience as {self.title} in {self.employer} in {self.city},{self.province}'
        s = datetime.strftime(self.start_date, '%Y-%m-%d')
        e = datetime.strftime(self.end_date, '%Y-%m-%d')
        s2 = f'{self.country},which started form {s} and ended on {e}'
        return s1 + s2


# s = '2005-05-31'
# e = '2012-07-30'
# we = WorkExperience(s, e, 'Marketing Manager', '0124', 'PTC', 'Shanghai', '', 'China')
# print(we())
# print(f'had worked for {we.years} years plus {we.months} months and {we.days} days')


class JobOffer:
    def __init__(self, employer, title, noc, hourly_wage, hours_per_week, city, province, country, start_date,
                 end_date):
        self.employer = employer
        self.noc = noc
        self.title = title
        self.hourly_wage = hourly_wage
        self.hours_per_week = hours_per_week
        self.city = city
        self.province = province
        self.country = country
        self.start_date = datetime.strptime(start_date, '%Y-%m-%d')
        self.end_date = datetime.strptime(end_date, '%Y-%m-%d')

        self.annual_wage = self.hourly_wage * self.hours_per_week * 52
        self.is_working = True if datetime.today() > self.start_date else False
        self.duration = relativedelta.relativedelta(self.end_date, self.start_date).years
        self.is_permanent = True if self.duration >= 100 else False

    def in_province(self, province):
        return True if self.province == province else False

    def __str__(self):
        return f'Job offer object (title:{self.title} in {self.employer})'

    def __call__(self, *args, **kwargs):
        s1 = f'It is a {self.duration}-year job offer as {self.title} in {self.employer} in {self.city} ,{self.province}'
        s = datetime.strftime(self.start_date, '%Y-%m-%d')
        e = datetime.strftime(self.end_date, '%Y-%m-%d')
        s2 = f'{self.country},which starts form {s} and ends on {e}'
        return s1 + s2


# s = '2020-03-01'
# e = '2120-03-01'
# jo = JobOffer('Svisa',"Immigration Consultant", '4166', 50, 30, 'Coquitlam', 'BC', 'Canada', s, e)
# print(jo.title+' '+jo.noc)
# print(jo())
# print(jo.is_permanent)

# Relative
class Relative:
    def __init__(self, relationship, province, country='Canada'):
        self.relationship = relationship
        self.province = province
        self.country = country

    def in_province(self, province):
        return True if self.province == province else False

    def __str__(self):
        return f'Object relative ({self.relationship} in {self.province})'

    def __call__(self, *args, **kwargs):
        return f'A relative as {self.relationship} in {self.province},{self.country}'


# r = Relative('sibling', 'SK')
# print(r.in_province('SK'))
# print(r())


class Spouse(Person):
    pass



# s = Spouse('May', 'Li', '1972-01-10', Sex.Female)
# print(s())


# adaptability
