from utils.language import *
from utils.NOC import *

# Education
edu_level_points_table = {
    'Doctor': 25,
    'Master': 23,
    'Professional': 23,
    # professional degree needed to practice in a licensed profession. https://www.canada.ca/en/immigration-refugees-citizenship/corporate/publications-manuals/operational-bulletins-manuals/permanent-residence/economic-classes/federal-skilled-workers/selection-criteria-education.html#after
    'Two more post-secondary': 22,  # Two or more post-secondary, at least one is more than 3 year
    '3-year post-secondary': 21,
    '2-year post-secondary': 19,
    '1-year post-secondary': 15,
    'Secondary': 5
}


def getEduPoints(education):
    return edu_level_points_table.get(education)


clb_level_points_talbe = {
    9: 6,
    8: 5,
    7: 4
}


def get1stLanguagePoints(language):

    clb,clbs=language.clb,[language.clb_r, language.clb_w, language.clb_l, language.clb_s]
    if clb < 7:
        print('the client is not eligible')
    elif clbs != []:
        points = []
        for clb in clbs:
            if clb > 9:
                clb = 9
            sub_points = clb_level_points_talbe.get(clb)
            points.append(sub_points)
        return points[0] + points[1] + points[2] + points[3]


# Second official language (maximum 4 points)
# You can get 4 points only if you have a score of at least CLB 5 in each of the 4 language abilities.
#
# Second official language	Points
# At least CLB 5 in all of the 4 abilities	4
# CLB 4 or less in any of the 4 abilities	0

def get2ndLanguagePoints(language):
    clb,clbs=language.clb,[language.clb_r, language.clb_w, language.clb_l, language.clb_s]
    if clbs[0] >= 5 & clbs[1] >= 5 & clbs[2] >= 5 & clbs[3] >= 5:
        return 4
    else:
        return 0


def getLanguagePoints(language):
    if len(language)== 0:
        return 0
    if language[0] is None:
        return 0
    elif language[1] is None:
        clb_points = get1stLanguagePoints(language[0])
    else:
        clb_points = get1stLanguagePoints(language[0]) + get2ndLanguagePoints(language[1])
    return clb_points


# work experience
# The Candidate's Canadian or international work experience must be:
#
# At least one year (1,560 hours), continuous full-time or an equal amount in continuous part-time;
# Paid work (volunteer work and unpaid internships do not count);
# In the same occupation;
# Within the last six years before the application to the NSOI and the application for permanent residence to IRCC; and
# In NOC 2011 Skill Type 0, Skill Level A or B.

we_points_table = {
    1: 9,
    2: 11,
    3: 11,
    4: 13,
    5: 13,
    6: 15
}


def getWePoints(year):
    if year > 6:
        year = 6
    if year < 1:
        print("work experience must be >=1 year")
        return None
    return we_points_table.get(year)


# Age
# Under 18 years	Ineligible
# 18-35 years	12
# 47  years and older	0

age_points_table = {
    36: 11,
    37: 10,
    38: 9,
    39: 8,
    40: 7,
    41: 6,
    42: 5,
    43: 4,
    44: 3,
    45: 2,
    46: 1
}


def getAgePoints(age):
    if age > 46:
        return 0

    if age < 18:
        return 0
    if age in range(18, 36):
        return 12
    else:
        return age_points_table.get(age)


# Adaptability
def getAdaptabilityPoints(factors):
    aeo_points = 5 if factors.get('AEO') else 0
    study_points = 5 if factors.get('Studied in NS') else 0
    spouse_study_points = 5 if factors.get('pouse studied in NS') else 0
    work_points = 5 if factors.get('Worked in NS') else 0
    spouse_work_points = 5 if factors.get('Spouse worked in NS') else 0
    relative_points = 5 if factors.get('Relative in NS') else 0
    spouse_language_points = 5 if factors.get('Spouse language') else 0

    points=aeo_points + study_points + spouse_study_points + work_points + spouse_work_points + relative_points
    sub_total=10 if points>10 else points

    in_detail = {
        'AEO': aeo_points,
        'Studied in NS': study_points,
        'Spouse studied in NS': spouse_study_points,
        'Worked in NS': work_points,
        'Spouse worked in NS': spouse_work_points,
        'Relative in NS': relative_points,
        'Spouse language': spouse_language_points
    }
    return sub_total, in_detail


def NSPoints(applicant, spouse=None):
    education_points = getEduPoints(applicant.ee_edu_level)
    language_points = getLanguagePoints(applicant.language)
    we_points = getWePoints(applicant.work_experience)
    age_points = getAgePoints(applicant.age)

    factors = {
        'AEO': applicant.aeo,
        'Studied in NS': applicant.studied_in_NS,
        'Spouse studied in NS': spouse.studied_in_NS,
        'Worked in NS': applicant.worked_in_NS,
        'Spouse worked in NS': spouse.worked_in_NS,
        'Relative in NS': applicant.relative_in_NS,
        'Spouse language': spouse.language_bonus

    }
    adaptability_points, adaptability_in_details = getAdaptabilityPoints(factors)

    total_points = education_points + language_points + we_points + age_points + adaptability_points
    points = {
        'total_points':total_points,
        'education_points': education_points,
        'language_points': language_points,
        'work_experience_points': we_points,
        'age_points': age_points,
        'adaptability_points': adaptability_points,
        'adaptability_detail': adaptability_in_details
    }

    return points
