from utils.language import *
from utils.NOC import *

# You need a minimum score of 60 points out of 100 to apply under a SINP International Skilled Worker (ISW)
# sub-category.

# I: LABOUR MARKET SUCCESS
# A: EDUCATION AND TRAINING

# Master’s or Doctorate degree, Canadian equivalency  23 Bachelor's degree OR
# a three or more year degree program at a university or college  20 Trade certification equivalent to journeyperson
# status in Saskatchewan  20 Canadian equivalency Diploma that requires two but less than three years at a
# university, college, trade or technical school, or other formal post-secondary institution  15 Canadian equivalency
# Certificate or at least two semesters but less than a two year program at a university, college, trade or technical
# school, or other formal post-secondary institution 12


edu_level_points_table = {
    'Doctor': 23,
    'Master': 23,
    '3-year post-secondary': 20,
    'Trade certification in Saskatchewan': 20,
    '2-year post-secondary': 15,
    '1-year post-secondary': 12  #
}


def getEduPoints(education):
    return edu_level_points_table.get(education)


# B: SKILLED WORK EXPERIENCE
# The work experience claimed must be relevant to the single occupation you've selected in your application.
# One year of work experience refers to 12 full months of work experience.

# a) In the 5 year period prior to application submission date
we_points_table5 = {
    5: 10,
    4: 8,
    3: 6,
    2: 4,
    1: 2
}
we_points_table6_10 = {
    5: 5,
    4: 4,
    3: 3,
    2: 2,
    1: 0
}


def getWePoints(years_within5, years_within6_10):
    if years_within5 > 5:
        years_within5 = 5
    if years_within5 < 1:
        we_points1 = 0
    else:
        we_point1 = we_points_table5.get(years_within5)

    if years_within6_10 > 5:
        years_within6_10 = 5
    if years_within6_10 < 1:
        we_points2 = 0
    else:
        we_point2 = we_points_table6_10.get(years_within6_10)

    return we_point1 + we_point2


# C: LANGUAGE ABILITY
language_points_table = {
    8: 20,
    7: 18,
    6: 16,
    5: 14,
    4: 12
}


def getLanguagePoints(clb):
    if clb > 8:
        clb = 8
    if clb < 4:
        return 0
    else:
        return language_points_table.get(clb)


# D: AGE
age_points_table = [
    [18, 21, 8],
    [22, 34, 12],
    [35, 45, 10],
    [46, 50, 8]
]


def getAgePoints(age):
    for row in age_points_table:
        if age in range(row[0], row[1] + 1):
            return row[2]

    return 0


# II: CONNECTION TO SASKATCHEWAN LABOUR MARKET & ADAPTABILITY

def getConnectionPoints(approved_joboffer, relative_in_SK, we_years_in_sk, se_years_in_sk):
    # has approved job offer

    joboffer_points = 30 if approved_joboffer else 0

    # has close family relative in SK The models or accompanying spouse has a family relative that is a Canadian
    # citizen or permanent resident living in Saskatchewan.
    # it includes direct relations below and step-family members or in-laws of the same relationships]

    relative_points = 20 if relative_in_SK else 0

    #  Past work experience in Saskatchewan 5
    # (A minimum of at least 12 months of work in the past five years on a valid work permit)

    we_in_sk_points = 5 if we_years_in_sk >= 1 else 0

    # Past student experience in Saskatchewan (A minimum of one full-time academic year of study at a recognized
    # Saskatchewan post-secondary education institution on a valid study permit) # 5

    se_in_sk_points = 5 if se_years_in_sk >= 1 else 0

    sub_total = joboffer_points + relative_points + we_in_sk_points + se_in_sk_points
    if sub_total > 30:
        sub_total = 30
    in_detail = {
        'Approved job offer points': joboffer_points,
        'Relative points': relative_points,
        'Work experience in SK': we_in_sk_points,
        'Study experience in SK': se_in_sk_points
    }
    return sub_total, in_detail


def SKPoints(applicant):
    edu_points = getEduPoints(applicant.ee_edu_level)
    we_points = getWePoints(applicant.we_within5, applicant.we_within6_10)
    language_points = getLanguagePoints(applicant.language[0].clb)
    age_points = getAgePoints(applicant.age)
    connection_points, in_detail_points = getConnectionPoints(applicant.has_approved_joboffer, applicant.relative_in_SK,
                                                              applicant.work_experience_in_SK,
                                                              applicant.stduy_experience_in_SK)

    total_points = edu_points + we_points + language_points + age_points + connection_points
    points = {
        'total_points': total_points,
        'education_points': edu_points,
        'work_experience_points': we_points,
        'language_points': language_points,
        'age_points': age_points,
        'connection_points': connection_points,
        'connection_points_in_detail': in_detail_points
    }

    return points
