# 1. IELTS->CLB level
# CLB Level:	Reading	Writing	Listening	Speaking
clb_ielts_table = (
    (10, 8.0, 7.5, 8.5, 7.5),
    (9, 7.0, 7.0, 8.0, 7.0),
    (8, 6.5, 6.5, 7.5, 6.5),
    (7, 6.0, 6.0, 6.0, 6.0),
    (6, 5.0, 5.5, 5.5, 5.5),
    (5, 4.0, 5.0, 5.0, 5.0),
    (4, 3.5, 4.0, 4.5, 4.0)
)


# 输入一组雅思成绩list：r w l s, 输出一个list对应CLB level： r w l s
def ielts_2_clbs(ielts):
    clb_level = []
    for column in range(1, 5):  # 从读到说，4个列，从1开始，开始找对应的clb级别
        factor = ielts[column - 1]  # r w l s每个子项
        for level in range(0, 7):  # 从CLB 级别10一直找到4 7行
            if factor > clb_ielts_table[0][column]:  # 如果语言成绩高于最高分，按照最高分算
                factor = clb_ielts_table[0][column]
            if factor < clb_ielts_table[6][column]:  # 如果低于最低分，则CLB Level =0 跳出本列循环
                clb_level.append(0)
                break
            # 要算每个子项的CLB值
            if clb_ielts_table[level][column] == ielts[column - 1]:
                clb_level.append(clb_ielts_table[level][0])  # 得到该雅思成绩对应的CLB 级别
                break
        # 如果某项分数不在阵列中
        for level in range(1, 7):
            if clb_ielts_table[level][column] < factor < clb_ielts_table[level - 1][column]:
                clb_level.append(clb_ielts_table[level][0])  # 得到该雅思成绩对应的CLB 级别
                break

    return clb_level


# 输入一组雅思成绩list：r w l s, 输出一个CLB level 数字
def ielts_2_clb(ielts):
    clb = 0
    for row in clb_ielts_table:
        if (ielts[0] >= row[1]) & (ielts[1] >= row[2]) & (ielts[2] >= row[3]) & (ielts[3] >= row[4]):
            clb = row[0]
            break
    return clb


# 2 CLPIP和CLB是一一对应关系。
# Canadian English Language Proficiency Index Program (CELPIP) – General Test score equivalency chart
# CLB Level	Reading	Writing	Listening	Speaking
# 10	10	10	10	10
# 9	9	9	9	9
# 8	8	8	8	8
# 7	7	7	7	7
# 6	6	6	6	6
# 5	5	5	5	5
# 4	4	4	4	4

def celpip_2_clb(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the CELPIP must be integer")
            has_error = True
    if not has_error:
        return min(rwls)
    else:
        return 0


def celpip_2_clbs(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the CELPIP must be integer")
            has_error = True
    if not has_error:
        return rwls
    else:
        return []


# 3 TEF
# Test d’évaluation de français pour le Canada (TEF Canada)
# Test score equivalency chart
# CLB Level	Reading	Writing	Listening	Speaking
# 10	    263-277	393-415	316-333	    393-415
# 9	        248-262	371-392	298-315	    371-392
# 8	        233-247	349-370	280-297	    349-370
# 7	        207-232	310-348	249-279	    310-348
# 6	        181-206	271-309	217-248	    271-309
# 5	        151-180	226-270	181-216	    226-270
# 4	        121-150	181-225	145-180	    181-225

tef_clb_table = (
    (10, 263, 277, 393, 415, 316, 333, 393, 415),
    (9, 248, 262, 371, 392, 298, 315, 371, 392),
    (8, 233, 247, 349, 370, 280, 297, 349, 370),
    (7, 207, 232, 310, 348, 249, 279, 310, 348),
    (6, 181, 206, 271, 309, 217, 248, 271, 309),
    (5, 151, 180, 226, 270, 181, 216, 226, 270),
    (4, 121, 150, 181, 225, 145, 180, 181, 225)
)


def tef_2_clb(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the TEF must be integer")
            has_error = True
    if not has_error:

        clb = []
        for column in range(0, 4):  # r w l s
            for row in range(0, 7):
                start = (column + 1) * 2 - 1
                end = (column + 1) * 2
                if rwls[column] in range(tef_clb_table[row][start], tef_clb_table[row][end] + 1):
                    clb.append(tef_clb_table[row][0])
                    break
        return min(clb)
    else:
        return 0


def tef_2_clbs(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the TEF must be integer")
            has_error = True
    if not has_error:
        clb = []
        for column in range(0, 4):  # r w l s
            for row in range(0, 7):
                start = (column + 1) * 2 - 1
                end = (column + 1) * 2
                if rwls[column] in range(tef_clb_table[row][start], tef_clb_table[row][end] + 1):
                    clb.append(tef_clb_table[row][0])
                    break
        return clb
    else:
        return 0


# 4 TCF
# Test de connaissance du français pour le Canada (TCF Canada)
# Test score equivalency chart
# CLB Level	    Reading	Writing	Listening	Speaking
# 10 and above	549-699	16-20	549-699	16-20
# 9	            524-548	14-15	523-548	14-15
# 8	            499-523	12-13	503-522	12-13
# 7	            453-498	10-11	458-502	10-11
# 6	            406-452	7-9	    398-457	7-9
# 5	            375-405	6	    369-397	6
# 4	            342-374	4-5	    331-368	4-5

tcf_clb_table = (
    (10, 549, 699, 16, 20, 549, 699, 16, 20),
    (9, 524, 548, 14, 15, 523, 548, 14, 15),
    (8, 499, 523, 12, 13, 503, 522, 12, 13),
    (7, 453, 498, 10, 11, 458, 502, 10, 11),
    (6, 406, 452, 7, 9, 398, 457, 7, 9),
    (5, 375, 405, 6, 6, 369, 397, 6, 6),
    (4, 342, 374, 4, 5, 331, 368, 4, 5)
)


def tcf_2_clb(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the TCF must be integer")
            has_error = True
    if not has_error:
        clb = []
        for column in range(0, 4):  # r w l s
            for row in range(0, 7):
                start = (column + 1) * 2 - 1
                end = (column + 1) * 2
                if rwls[column] in range(tcf_clb_table[row][start], tcf_clb_table[row][end] + 1):
                    clb.append(tcf_clb_table[row][0])
                    break
        return min(clb)
    else:
        return 0


def tcf_2_clbs(rwls):
    has_error = False
    for s in rwls:
        if not isinstance(s, int):
            print("the TCF must be integer")
            has_error = True
    if not has_error:
        clb = []
        for column in range(0, 4):  # r w l s
            for row in range(0, 7):
                start = (column + 1) * 2 - 1
                end = (column + 1) * 2
                if rwls[column] in range(tcf_clb_table[row][start], tcf_clb_table[row][end] + 1):
                    clb.append(tcf_clb_table[row][0])
                    break
        return clb
    else:
        return 0


def getCLBs(rwls, format):
    if format == 'IELTS':
        clb = ielts_2_clb(rwls)
        clbs = ielts_2_clbs(rwls)
    if format == 'CELPIP':
        clb = celpip_2_clb(rwls)
        clbs = celpip_2_clbs(rwls)
    if format == 'TEF':
        clb = tef_2_clb(rwls)
        clbs = tef_2_clbs(rwls)
    if format == 'TCF':
        clb = tcf_2_clb(rwls)
        clbs = tcf_2_clbs(rwls)
    return clb, clbs
